@@ -98,6 +98,13 @@ impl<T> Cell<T> {
     ///
     /// Returns Err if the cell is refered or in critical section.
     pub fn try_take(&self) -> Result<Option<T>, Error<()>> {
+        if self.will_drop.load(SeqCst) {
+            return Err(Error {
+                state: MemoryState::Erasing,
+                input: (),
+                retry: true,
+            });
+        }
         let _cs = CriticalSection::new();
         if let Err(state) = self.state.fetch_update(SeqCst, SeqCst, |x| {
             if x.ref_num() == Ok(0) {
@@ -118,7 +125,6 @@ impl<T> Cell<T> {
             }
         } else {
             let ret = unsafe { ptr::read(self.ptr()) };
-            self.will_drop.store(false, SeqCst);
             self.state.store(MemoryState::Uninitialized.into(), SeqCst);
             Ok(Some(ret))
         }
@@ -173,6 +179,13 @@ impl<T> Cell<T> {
     ///
     /// Returns Err if the value is refered, initialized or in critical section.
     pub fn try_set(&self, value: T) -> Result<(), Error<T>> {
+        if self.will_drop.load(SeqCst) {
+            return Err(Error {
+                state: MemoryState::Erasing,
+                input: value,
+                retry: true,
+            });
+        }
         let _cs = CriticalSection::new();
         if let Err(state) = self.state.compare_exchange(
             MemoryState::Uninitialized.into(),
@@ -207,61 +220,62 @@ impl<T> Cell<T> {
     /// Notice: `Spin`
     pub fn fetch_update<F>(&self, mut f: F) -> Result<Option<T>, Option<Ref<T>>>
     where
-        F: FnMut(&Option<Ref<T>>) -> Option<Option<T>>,
+        F: FnMut(Option<&Ref<T>>) -> Option<Option<T>>,
     {
         let now = self.get().map_or_else(|_| None, |x| Some(x));
-        let ret = match f(&now) {
+        self.will_drop.store(true, SeqCst);
+        let ret = match f(now.as_ref()) {
             Some(new) => {
-                if let Some(now) = now {
-                    now.remove();
-                }
-                let _cs = CriticalSection::new();
                 let old = retry(
-                    |new| match self.state.fetch_update(SeqCst, SeqCst, |x| {
-                        if x.state().is_uninitialized() || x.ref_num() == Ok(0) {
-                            Some(
-                                if new.is_some() {
-                                    MemoryState::Initializing
-                                } else {
-                                    MemoryState::Erasing
-                                }
-                                .into(),
-                            )
-                        } else {
-                            None
-                        }
-                    }) {
-                        Ok(state) => {
-                            let ret = if state.state().is_uninitialized() {
+                    |new| {
+                        let _cs = CriticalSection::new();
+                        match self.state.fetch_update(SeqCst, SeqCst, |x| {
+                            if x.state().is_uninitialized() || x.ref_num() <= Ok(1) {
+                                Some(
+                                    if new.is_some() {
+                                        MemoryState::Initializing
+                                    } else {
+                                        MemoryState::Erasing
+                                    }
+                                    .into(),
+                                )
+                            } else {
                                 None
-                            } else {
-                                Some(unsafe { ptr::read(self.ptr()) })
-                            };
-                            self.will_drop.store(false, SeqCst);
-                            if let Some(new) = new {
-                                unsafe { ptr::write(self.ptr(), new) };
-                                self.state.store(MemoryState::Initialized.into(), SeqCst);
-                            } else {
-                                self.state.store(MemoryState::Uninitialized.into(), SeqCst);
                             }
-                            Ok(ret)
-                        }
-                        Err(state) => {
-                            let state = state.state();
-                            Err(Error {
-                                state,
-                                input: new,
-                                retry: true,
-                            })
+                        }) {
+                            Ok(state) => {
+                                let ret = if state.state().is_uninitialized() {
+                                    None
+                                } else {
+                                    Some(unsafe { ptr::read(self.ptr()) })
+                                };
+                                if let Some(new) = new {
+                                    unsafe { ptr::write(self.ptr(), new) };
+                                    self.state.store(MemoryState::Initialized.into(), SeqCst);
+                                } else {
+                                    self.state.store(MemoryState::Uninitialized.into(), SeqCst);
+                                }
+                                Ok(ret)
+                            }
+                            Err(state) => {
+                                let state = state.state();
+                                Err(Error {
+                                    state,
+                                    input: new,
+                                    retry: true,
+                                })
+                            }
                         }
                     },
                     new,
                 )
                 .unwrap();
+                core::mem::forget(now);
                 Ok(old)
             }
             None => Err(now),
         };
+        self.will_drop.store(false, SeqCst);
         ret
     }
 
@@ -269,6 +283,13 @@ impl<T> Cell<T> {
     ///
     /// Returns Err if the value is refered or in critical section.
     pub fn try_replace(&self, value: T) -> Result<Option<T>, Error<T>> {
+        if self.will_drop.load(SeqCst) {
+            return Err(Error {
+                state: MemoryState::Erasing,
+                input: value,
+                retry: true,
+            });
+        }
         let _cs = CriticalSection::new();
         match self.state.fetch_update(SeqCst, SeqCst, |x| {
             if x.state().is_uninitialized() || x.ref_num() == Ok(0) {
@@ -283,7 +304,6 @@ impl<T> Cell<T> {
                 } else {
                     Some(unsafe { ptr::read(self.ptr()) })
                 };
-                self.will_drop.store(false, SeqCst);
                 unsafe { ptr::write(self.ptr(), value) };
                 self.state.store(MemoryState::Initialized.into(), SeqCst);
                 Ok(ret)
@@ -311,6 +331,13 @@ impl<T> Cell<T> {
     ///
     /// Returns Err if the cell is in critical section.
     pub fn get_or_try_init(&self, value: T) -> Result<Ref<T>, Error<T>> {
+        if self.will_drop.load(SeqCst) {
+            return Err(Error {
+                state: MemoryState::Erasing,
+                input: value,
+                retry: true,
+            });
+        }
         let _cs = CriticalSection::new();
         if let Err(_) = self.state.compare_exchange(
             MemoryState::Uninitialized.into(),
