@@ -21,8 +21,8 @@ fn test() {
     let mut handle = Vec::new();
     for _ in TEST_DATA {
         handle.push(thread::spawn(move || loop {
-            let result = CELL.take().unwrap();
-            if let Some(_) = result {
+            let result = CELL.take();
+            if let Ok(Some(_)) = result {
                 break;
             } else {
                 thread::yield_now();
